@@ -69,16 +69,14 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	#sglobCoefNames <- fixCoefNames(names(coeffs(global.model)))
 
 	n.vars <- length(allTerms)
-	ret <- numeric(0L)
-	formulas <- character(0L)
 
 	if(isTRUE(rankArgs$REML) || (isTRUE(.isREMLFit(global.model)) && is.null(rankArgs$REML)))
-		warning("comparing models with different fixed effects fitted by REML")
+		warning("comparing models fitted by REML")
 
 	if (beta && is.null(tryCatch(beta.weights(global.model), error=function(e) NULL,
 		warning=function(e) NULL))) {
 		warning("do not know how to calculate B-weights for ",
-				class(global.model)[1L], ", argument 'beta' ignored")
+				sQuote(class(global.model)[1L]), ", argument 'beta' ignored")
 		beta <- FALSE
 	}
 
@@ -139,10 +137,14 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 			extraNames <- ifelse(names(extra) != "", names(extra), extraNames)
 
 		extra <- structure(as.list(unique(extra)), names = extraNames)
-		if("R^2" %in% extra) {
+
+		if(any(c("adjR^2", "R^2") %in% extra)) {
 			null.fit <- null.fit(global.model, TRUE, gmFormulaEnv)
 			extra[extra == "R^2"][[1L]] <- function(x) r.squaredLR(x, null.fit)
+			extra[extra == "adjR^2"][[1L]] <-
+				function(x) attr(r.squaredLR(x, null.fit), "adj.r.squared")
 		}
+
 		extra <- sapply(extra, match.fun, simplify = FALSE)
 		applyExtras <- function(x) unlist(lapply(extra, function(f) f(x)))
 		extraResult <- applyExtras(global.model)
@@ -299,6 +301,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 		} # for (ivar ...)
 	} ### for (j ...)
 
+	names(calls) <- ord
 	if(!evaluate) return(calls[seq.int(k)])
 
 	if(k < nrow(ret)) ret <- ret[seq.int(k), , drop=FALSE]
@@ -353,7 +356,6 @@ function(x, subset, select, recalc.weights = TRUE, ...) {
 	if (missing(select)) {
 		if(missing(subset)) return(x)
 		e <- .substHas(substitute(subset))
-		#print(e)
 		i <- eval(e, x, parent.frame())
 		return(`[.model.selection`(x, i, recalc.weights = recalc.weights, ...))
 	} else {
@@ -371,13 +373,17 @@ function(x, subset, select, recalc.weights = TRUE, ...) {
 function (x, i, j, recalc.weights = TRUE, ...) {
 	ret <- `[.data.frame`(x, i, j, ...)
 	if (missing(j)) {
-		att <- attributes(x)
 		s <- c("row.names", "calls")
-		att[s] <- lapply(att[s], `[`, i)
-		attributes(ret) <- att
+		k <- match(dimnames(ret)[[1L]], dimnames(x)[[1L]])
+		attrib <- attributes(x)
+		attrib[s] <- lapply(attrib[s], `[`, k)
+		attributes(ret) <- attrib
 		if(recalc.weights)
 			ret$weight <- ret$weight / sum(ret$weight)
 			#ret[, 'weight'] <- ret[, 'weight'] / sum(ret[, 'weight'])
+
+		if(!is.null(warningList <- attr(ret, "warnings")))
+			attr(ret, "warnings") <- warningList[sapply(warningList, attr, "id") %in% rownames(ret)]
 	} else {
 		cls <- class(ret)
 		class(ret) <- cls[cls != "model.selection"] # numeric or data.frame
@@ -386,23 +392,17 @@ function (x, i, j, recalc.weights = TRUE, ...) {
 }
 
 `print.model.selection` <-
-function(x, abbrev.names = TRUE, ...) {
+function(x, abbrev.names = TRUE, warnings = getOption("warn") != -1L, ...) {
 	if(!is.null(x$weight))
 		x$weight <- round(x$weight, 3L)
-
 	xterms <- attr(x, "terms")
-
 	if(is.null(xterms)) {
 		print.data.frame(x, ...)
 	} else {
-
 		xterms <- gsub(" ", "", xterms)
-
-		if(abbrev.names)
-			xterms <- abbreviateTerms(xterms, 3L)
+		if(abbrev.names) xterms <- abbreviateTerms(xterms, 3L)
 
 		colnames(x)[seq_along(xterms)] <-  xterms
-
 		cl <- attr(x, "global.call")
 		if(!is.null(cl)) {
 			cat("Global model call: ")
@@ -410,24 +410,24 @@ function(x, abbrev.names = TRUE, ...) {
 			cat("---\n")
 		}
 
-		cat ("Model selection table \n")
+		cat("Model selection table \n")
 		dig <- c("R^2" = 4L, df = 0, logLik = 3, AICc = 1, AICc = 1, AIC = 1,
 			BIC = 1, QAIC = 1, QAICc = 1, ICOMP = 1, Cp = 1, delta = 2L, weight = 3L)
 
 		j <- match(colnames(x), names(dig), nomatch = 0)
-
 		i <- sapply(x, is.numeric) & (j == 0L)
 		x[, i] <- signif(x[, i], 4L)
+		for(i in names(dig)[j]) x[, i] <- round(x[, i], digits = dig[i])
 
-		for(i in names(dig)[j])
-			x[, i] <- round(x[, i], digits = dig[i])
-
-		print.default(as.matrix(x[, !sapply(x, function(.x) all(is.na(.x)))]),
-					  na.print="", quote=FALSE)
+		print.default(as.matrix(x)[, !sapply(x, function(.x) all(is.na(.x))),
+			drop = FALSE], na.print = "", quote = FALSE)
 		if (!is.null(attr(x, "random.terms"))) {
 			cat("Random terms:", paste(attr(x, "random.terms"), collapse=", "),
 				"\n")
 		}
+		if (warnings && !is.null(attr(x, "warnings"))) {
+			cat("\n"); print.warnings(attr(x, "warnings"))
+		}
 	}
 }
 
